@@ -7,7 +7,7 @@
 #ifndef AUX_BITMAP_H
 #define AUX_BITMAP_H
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 #include <aal/libaal.h>
 
 #define AUX_BITMAP_MAGIC	"R4BtMp"
@@ -64,7 +64,6 @@ extern aux_bitmap_t *aux_bitmap_clone(aux_bitmap_t *bitmap);
 
 extern void aux_bitmap_resize(aux_bitmap_t *bitmap, uint64_t len);
 extern void aux_bitmap_close(aux_bitmap_t *bitmap);
-extern char *aux_bitmap_map(aux_bitmap_t *bitmap);
 
 extern aux_bitmap_t *aux_bitmap_unpack(aal_stream_t *stream);
 extern errno_t aux_bitmap_pack(aux_bitmap_t *bitmap, 
