@@ -10,8 +10,6 @@
 
 extern void reiser4_fs_close(reiser4_fs_t *fs);
 
-extern errno_t reiser4_fs_root_key(reiser4_fs_t *fs, reiser4_key_t *key);
-
 #ifndef ENABLE_MINIMAL
 extern reiser4_fs_t *reiser4_fs_open(aal_device_t *device, bool_t check);
 #else
@@ -20,14 +18,6 @@ extern reiser4_fs_t *reiser4_fs_open(aal_device_t *device);
 
 #ifndef ENABLE_MINIMAL
 
-#define MASTER_PACK_SIGN "MSTR"
-#define STATUS_PACK_SIGN "STAT"
-#define BACKUP_PACK_SIGN "BCKP"
-#define FORMAT_PACK_SIGN "FRMT"
-#define ALLOC_PACK_SIGN  "ALLO"
-#define NODE_PACK_SIGN   "NODE"
-#define BLOCK_PACK_SIGN  "BLCK"
-
 extern errno_t reiser4_fs_sync(reiser4_fs_t *fs);
 
 extern errno_t reiser4_fs_clobber(aal_device_t *device);
@@ -42,12 +32,14 @@ extern errno_t reiser4_fs_layout(reiser4_fs_t *fs,
 
 extern reiser4_owner_t reiser4_fs_belongs(reiser4_fs_t *fs, blk_t blk);
 
-extern errno_t reiser4_fs_check_len(reiser4_fs_t *fs, count_t blocks);
+extern errno_t reiser4_fs_check_len(aal_device_t *device, 
+				    uint32_t blksize, 
+				    count_t blocks);
 
 extern reiser4_fs_t *reiser4_fs_create(aal_device_t *device,
 				       fs_hint_t *hint);
 
-extern errno_t reiser4_fs_backup(reiser4_fs_t *fs, aal_stream_t *stream);
+extern errno_t reiser4_fs_backup(reiser4_fs_t *fs, backup_hint_t *hint);
 
 #endif
 
