@@ -120,7 +120,7 @@ enum reiser4_item_plug_id {
 	ITEM_NODEPTR40_ID	= 0x3,
 	ITEM_ACL40_ID		= 0x4,
 	ITEM_EXTENT40_ID	= 0x5,
-	ITEM_TAIL40_ID		= 0x6,
+	ITEM_PLAIN40_ID		= 0x6,
 	ITEM_CTAIL40_ID		= 0x7,
 	ITEM_BLACKBOX40_ID	= 0x8,
 	ITEM_LAST_ID
@@ -135,6 +135,8 @@ enum reiser4_item_group {
 	EXTENT_ITEM		= 0x4,
 	PERMISSION_ITEM		= 0x5, /* not used yet */
 	SAFE_LINK_ITEM		= 0x6,
+	CTAIL_ITEM		= 0x7,
+	BLACK_BOX_ITEM		= 0x8,
 	LAST_ITEM
 };
 
@@ -331,6 +333,7 @@ typedef struct reiser4_node reiser4_node_t;
 typedef struct reiser4_place reiser4_place_t;
 
 #define place_blknr(place) ((place)->node->block->nr)
+#define place_blksize(place) ((place)->node->block->size)
 
 /* Tree coord struct. */
 struct reiser4_place {
@@ -344,6 +347,11 @@ struct reiser4_place {
 	   method fetch() and stored here to make work with them simpler. */
 	void *body;
 	uint32_t len;
+
+	/* First unit offset within the item. Used by item internal code only. 
+	   Useful to minimize the code (e.g. tail, ctail). */
+	uint32_t off;
+	
 	reiser4_key_t key;
 	reiser4_plug_t *plug;
 };
@@ -425,7 +433,8 @@ enum shift_flags {
 	   shift. */
 	SF_MOVE_POINT    = 1 << 2,
 
-	/* Allows to update insert point during shift. */
+	/* Care about the insert point, shift only till this point. If not 
+	   given, as much data as fit the destination node will be shifted. */
 	SF_UPDATE_POINT  = 1 << 3,
 
 	/* Controls if shift allowed to merge border items or only whole items
@@ -438,7 +447,10 @@ enum shift_flags {
 	   in existent nodes (one insert point points to and its neighbours)*/
 	SF_ALLOW_ALLOC   = 1 << 5,
 
-	SF_ALLOW_PACK    = 1 << 6
+	SF_ALLOW_PACK    = 1 << 6,
+
+	/* Hold (do not leave) the current position (pointed by place->pos). */
+	SF_HOLD_POS      = 1 << 7
 };
 
 typedef enum shift_flags shift_flags_t;
@@ -725,15 +737,12 @@ typedef struct trans_hint trans_hint_t;
 
 /* This structure contains related to tail conversion. */
 struct conv_hint {
-	/* Chuck size to be used for tail convertion. */
-	uint32_t chunk;
-	
 	/* New bytes value */
 	uint64_t bytes;
 
 	/* Bytes to be converted. */
 	uint64_t count;
-
+	
 	/* File will be converted starting from this key. */
 	reiser4_key_t offset;
 	
@@ -803,7 +812,7 @@ typedef struct repair_hint repair_hint_t;
 
 enum reiser4_backuper {
 	BK_MASTER	= 0x0,
-	BK_FORMAT	= 0X1,
+	BK_FORMAT	= 0x1,
 	BK_LAST		= 0x2
 };
 
@@ -872,9 +881,6 @@ struct reiser4_key_ops {
 	/* Compares two keys by comparing locality and objectid. */
 	int (*compshort) (reiser4_key_t *, reiser4_key_t *);
 	
-	/* Copyies src key to dst one */
-	errno_t (*assign) (reiser4_key_t *, reiser4_key_t *);
-	
 	/* Builds generic key (statdata, file body, etc). That is build key by
 	   all its components. */
 	errno_t (*build_generic) (reiser4_key_t *, key_type_t,
@@ -1040,9 +1046,9 @@ struct item_balance_ops {
 			    lookup_bias_t);
 	
 #ifndef ENABLE_MINIMAL
-	/* Fuses two neighbour items in the same node. Returns space released
+	/* Merges two neighbour items in the same node. Returns space released
 	   Needed for fsck. */
-	int32_t (*fuse) (reiser4_place_t *, reiser4_place_t *);
+	int32_t (*merge) (reiser4_place_t *, reiser4_place_t *);
 	
 	/* Checks if items mergeable, that is if unit of one item can belong to
 	   another one. Returns 1 if so, 0 otherwise. */
@@ -1152,6 +1158,9 @@ struct item_debug_ops {
 typedef struct item_debug_ops item_debug_ops_t;
 
 struct item_tree_ops {
+	/* Initialize the item-specific place info. */
+	void (*init) (reiser4_place_t *);
+	
 	/* Return block number from passed place. Place is nodeptr item. */
 	blk_t (*down_link) (reiser4_place_t *);
 
@@ -1210,7 +1219,7 @@ struct reiser4_node_ops {
 			  shift_hint_t *);
 
 	/* Fuses two neighbour items in passed node at passed positions. */
-	errno_t (*fuse) (reiser4_node_t *, pos_t *, pos_t *);
+	errno_t (*merge) (reiser4_node_t *, pos_t *, pos_t *);
     
 	/* Checks thoroughly the node structure and fixes what needed. */
 	errno_t (*check_struct) (reiser4_node_t *, uint8_t);
@@ -1741,7 +1750,8 @@ struct tree_ops {
 	/* Get the safe link locality. */
 	uint64_t (*slink_locality) (tree_entity_t *);
 
-	/* decriment the free block count in the format. */
+	/* increment/decriment the free block count in the format. */
+	errno_t (*inc_free) (tree_entity_t *, count_t);
 	errno_t (*dec_free) (tree_entity_t *, count_t);
 #endif
 	/* Returns the next item. */
