@@ -17,6 +17,14 @@ extern char *reiser4_print_key(reiser4_key_t *key);
 
 extern char *reiser4_print_inode(reiser4_key_t *key);
 
+#ifdef ENABLE_DEBUG
+
+extern void reiser4_print_format(reiser4_format_t *format, uint16_t options);
+
+extern void reiser4_print_node(reiser4_node_t *node, uint32_t start, 
+			       uint32_t count, uint16_t options);
+
 #endif
 
 #endif
+#endif
