@@ -13,7 +13,9 @@
 /* Statistics gathered during the pass. */
 typedef struct repair_am_stat {
 	uint64_t read_leaves, by_leaf, by_item_leaves;
-	uint64_t read_twigs,  by_twig, by_item_twigs;
+	uint64_t read_twigs,  by_twig, by_item_twigs, empty;
+
+	uint64_t *files;
 	time_t time;
 } repair_am_stat_t;
 
@@ -25,9 +27,8 @@ typedef struct repair_am {
 	aux_bitmap_t *bm_twig;
 	aux_bitmap_t *bm_leaf;
 	
-	repair_progress_handler_t *progress_handler;    
-	repair_progress_t *progress;
 	repair_am_stat_t stat;
+	aal_gauge_t *gauge;
 } repair_am_t;
 
 extern errno_t repair_add_missing(repair_am_t *am);
