@@ -9,7 +9,8 @@
 
 #include <reiser4/filesystem.h>
 
-extern errno_t repair_format_open(reiser4_fs_t *fs, uint8_t mode);
+extern errno_t repair_format_check_struct(reiser4_fs_t *fs, 
+					  uint8_t mode, uint32_t options);
 
 extern errno_t repair_format_update(reiser4_format_t *format);
 
@@ -19,6 +20,10 @@ extern errno_t repair_format_pack(reiser4_format_t *format,
 extern reiser4_format_t *repair_format_unpack(reiser4_fs_t *fs,
 					      aal_stream_t *stream);
 
-extern void repair_format_print(reiser4_format_t *format, aal_stream_t *stream);
+extern void repair_format_print(reiser4_format_t *format, 
+				aal_stream_t *stream);
+
+extern errno_t repair_format_check_backup(aal_device_t *device, 
+					  backup_hint_t *hint);
 
 #endif
