@@ -9,7 +9,9 @@
 
 #include <repair/repair.h>
 
-extern errno_t repair_master_open(reiser4_fs_t *fs, uint8_t mode);
+extern errno_t repair_master_check_struct(reiser4_fs_t *fs, 
+					  uint8_t mode,
+					  uint32_t options);
 
 extern errno_t repair_master_pack(reiser4_master_t *master,
 				  aal_stream_t *stream);
@@ -21,4 +23,6 @@ extern void repair_master_print(reiser4_master_t *master,
 				aal_stream_t *stream,
 				uuid_unparse_t unparse);
 
+extern errno_t repair_master_check_backup(backup_hint_t *hint);
+
 #endif
