@@ -14,7 +14,8 @@
 enum {
 	REPAIR_DEBUG	= 0x0,
 	REPAIR_WHOLE	= 0x1,
-	REPAIR_YES	= 0X2,
+	REPAIR_NO_MKID	= 0x2,
+	REPAIR_YES	= 0X3,
 	REPAIR_LAST
 };
 
@@ -35,10 +36,10 @@ extern errno_t repair_check(repair_data_t *repair);
 
 #define repair_error_count(repair, error)		\
 ({							\
-	if (res > 0) {					\
+	if (error > 0) {					\
 		if (error & RE_FATAL)			\
 			repair->fatal++;		\
-		else if (res & RE_FIXABLE)		\
+		else if (error & RE_FIXABLE)		\
 			repair->fixable++;		\
 	}						\
 })
