@@ -19,7 +19,7 @@ int64_t reiser4_flow_read(reiser4_tree_t *tree, trans_hint_t *hint) {
 	aal_assert("umka-2510", hint != NULL);
 	
 	buff = hint->specific;
-	reiser4_key_assign(&key, &hint->offset);
+	aal_memcpy(&key, &hint->offset, sizeof(key));
 
 #ifndef ENABLE_MINIMAL
 	hint->blocks = tree->blocks;
@@ -102,7 +102,7 @@ int64_t reiser4_flow_read(reiser4_tree_t *tree, trans_hint_t *hint) {
 	}
 
 	hint->specific = buff;
-	reiser4_key_assign(&hint->offset, &key);
+	aal_memcpy(&hint->offset, &key, sizeof(key));
 	
 	return total;
 }
@@ -129,7 +129,7 @@ int64_t reiser4_flow_write(reiser4_tree_t *tree, trans_hint_t *hint) {
 	aal_assert("umka-2507", hint != NULL);
 	
 	buff = hint->specific;
-	reiser4_key_assign(&key, &hint->offset);
+	aal_memcpy(&key, &hint->offset, sizeof(key));
 
 	hint->blocks = tree->blocks;
 
@@ -182,7 +182,7 @@ int64_t reiser4_flow_write(reiser4_tree_t *tree, trans_hint_t *hint) {
 
 	hint->bytes = bytes;
 	hint->specific = buff;
-	reiser4_key_assign(&hint->offset, &key);
+	aal_memcpy(&hint->offset, &key, sizeof(key));
 	
 	return total;
 }
@@ -201,7 +201,7 @@ int64_t reiser4_flow_truncate(reiser4_tree_t *tree, trans_hint_t *hint) {
 	aal_assert("umka-2475", tree != NULL);
 	aal_assert("umka-2476", hint != NULL);
 
-	reiser4_key_assign(&key, &hint->offset);
+	aal_memcpy(&key, &hint->offset, sizeof(key));
 
 	/* Setting up region func to release region callback. It is needed for
 	   releasing extent blocks. */
@@ -319,7 +319,7 @@ int64_t reiser4_flow_truncate(reiser4_tree_t *tree, trans_hint_t *hint) {
 	
 	hint->bytes = bytes;
 
-	reiser4_key_assign(&hint->offset, &key);
+	aal_memcpy(&hint->offset, &key, sizeof(key));
 	return total;
 }
 
@@ -331,6 +331,7 @@ errno_t reiser4_flow_convert(reiser4_tree_t *tree, conv_hint_t *hint) {
 	errno_t res;
 	int64_t conv;
 	uint64_t size;
+	int64_t insert;
 	uint32_t blksize;
 	trans_hint_t trans;
 	
@@ -339,31 +340,33 @@ errno_t reiser4_flow_convert(reiser4_tree_t *tree, conv_hint_t *hint) {
 	aal_assert("umka-2481", hint->plug != NULL);
 
 	blksize = reiser4_tree_get_blksize(tree);
-	reiser4_key_assign(&trans.offset, &hint->offset);
+	aal_memcpy(&trans.offset, &hint->offset, sizeof(trans.offset));
 
-	/* Check if convertion chunk is zero. If so -- use filesystem block
-	   size. */
-	if (hint->chunk == 0)
-		hint->chunk = blksize;
+	insert = hint->count;
+	
+	/* Check if the start byte is not multiple by the block size. 
+	   Adjust if needed. */
+	conv = reiser4_key_get_offset(&trans.offset);
+	size = conv & (blksize - 1);
+	
+	if (size) {
+		reiser4_key_set_offset(&trans.offset, conv - size);
+		if (hint->count != MAX_UINT64)
+			insert += size;
+	}
 
 	/* Check if number of bytes to be converted is not multiple of block
-	   size and this is tail2extent conversion. If so, have to align
-	   @hint->count byblock size into highest side. */
-	if (hint->plug->id.group == EXTENT_ITEM && 
-	    hint->count != MAX_UINT64 && (hint->count & (blksize - 1)) != 0)
-	{
-		hint->count += blksize -
-			(hint->count & (blksize - 1));
-	}
+	   size. If so, have to round @hint->count up to blksize. */
+	size = insert & (blksize - 1);
+	
+	if (hint->count != MAX_UINT64 && size)
+		insert += (blksize - size);
 	
 	/* Loop until @size bytes is converted. */
-	for (size = hint->count, hint->bytes = 0;
-	     size > 0; size -= conv)
-	{
+	for (hint->bytes = 0; insert > 0; insert -= conv) {
 		/* Each convertion tick may be divided onto tree stages:
 
-		   (1) Read convert chunk (@hint->chunk bytes long now) to
-		   @trans hint.
+		   (1) Read blksize bytes @trans hint.
 
 		   (2) Truncate data in tree we have just read described by
 		   @trans hint.
@@ -374,10 +377,7 @@ errno_t reiser4_flow_convert(reiser4_tree_t *tree, conv_hint_t *hint) {
 		*/
 		
 		/* Preparing buffer to read data to it. */
-		trans.count = hint->chunk;
-
-		if (trans.count > size)
-			trans.count = size;
+		trans.count = blksize > insert ? insert : blksize;
 
 		if (!(buff = aal_calloc(trans.count, 0)))
 			return -ENOMEM;
@@ -396,8 +396,10 @@ errno_t reiser4_flow_convert(reiser4_tree_t *tree, conv_hint_t *hint) {
 		if (conv == 0) {
 			/* If nothing was read, depending on hint->ins_hole flag
 			   the hole will be inserted or convertion is finished. */
-			if (!hint->ins_hole)
-				break;
+			if (!hint->ins_hole) {
+				res = 0;
+				goto error_free_buff;
+			}
 
 			trans.specific = NULL;
 		} else {
@@ -410,16 +412,17 @@ errno_t reiser4_flow_convert(reiser4_tree_t *tree, conv_hint_t *hint) {
 			goto error_free_buff;
 		}
 
-		trans.count = conv;
-		trans.plug = hint->plug;
-		trans.shift_flags = SF_DEFAULT;
-		trans.place_func = hint->place_func;
-		
-		/* Third stage -- writing data back to tree with new item plugin
-		   used.*/
-		if ((conv = reiser4_flow_write(tree, &trans)) < 0) {
-			res = conv;
-			goto error_free_buff;
+		if (insert > 0) {
+			/* Insert only allowed amount of bytes. */
+			trans.count = conv > insert ? insert : conv;
+			trans.plug = hint->plug;
+			trans.shift_flags = SF_DEFAULT;
+			trans.place_func = hint->place_func;
+
+			/* Third stage -- writing data back to tree with 
+			   new item plugin used.*/
+			if ((res = reiser4_flow_write(tree, &trans)) < 0)
+				goto error_free_buff;
 		}
 
 		hint->bytes += trans.bytes;
