@@ -8,9 +8,7 @@
 
 /* Callback for item_ops->layout method to mark all the blocks, items points 
    to, in the allocator. */
-static errno_t callback_item_mark_region(void *object, uint64_t start, 
-					 uint64_t count, void *data)
-{
+static errno_t cb_item_mark_region(uint64_t start, uint64_t count, void *data) {
 	repair_am_t *am = (repair_am_t *)data;
 	
 	aal_assert("vpf-735", data != NULL);
@@ -28,7 +26,7 @@ static errno_t callback_item_mark_region(void *object, uint64_t start,
 /* Callback for traverse through all items of the node. Calls for the item, 
    determined by place, layout method, if it is not the branch and has pointers
    to some blocks. */
-static errno_t callback_layout(reiser4_place_t *place, void *data) {
+static errno_t cb_layout(reiser4_place_t *place, void *data) {
 	aal_assert("vpf-649", place != NULL);
 	aal_assert("vpf-748", !reiser4_item_branch(place->plug));
 
@@ -38,26 +36,7 @@ static errno_t callback_layout(reiser4_place_t *place, void *data) {
 	/* All these blocks should not be used in the allocator and should be 
 	   forbidden for allocation. Check it somehow first. */
 	return plug_call(place->plug->o.item_ops->object, layout,
-			 place, callback_item_mark_region, data);
-}
-
-static void repair_add_missing_setup(repair_am_t *am) {
-	aal_assert("vpf-887", am != NULL);
-	
-	aal_memset(am->progress, 0, sizeof(*am->progress));
-	
-	if (!am->progress_handler)
-		return;
-	
-	am->progress->type = GAUGE_PERCENTAGE;
-	
-	am->progress->text = "***** AddMissing Pass: inserting unconnected "
-		"nodes into the tree.";
-	
-	am->progress->state = PROGRESS_START;
-	time(&am->stat.time);
-	am->progress_handler(am->progress);
-	am->progress->text = NULL;
+			 place, cb_item_mark_region, data);
 }
 
 static void repair_add_missing_update(repair_am_t *am) {
@@ -67,19 +46,17 @@ static void repair_add_missing_update(repair_am_t *am) {
 	
 	aal_assert("vpf-886", am != NULL);
 	
-	if (!am->progress_handler)
-		return;
-	
 	stat = &am->stat;
 	
 	aal_stream_init(&stream, NULL, &memory_stream);
 	
-	aal_stream_format(&stream, "\tTwigs: read %llu, inserted %llu, by "
-			  "items %llu\n", stat->read_twigs, stat->by_twig,
-			  stat->by_item_twigs);
-	    
+	aal_stream_format(&stream, "\tTwigs: read %llu, inserted %llu, "
+			  "by item %llu, empty %llu\n", stat->read_twigs, 
+			  stat->by_twig, stat->by_item_twigs,
+			  stat->empty);
+	
 	aal_stream_format(&stream, "\tLeaves: read %llu, inserted %llu, by "
-			  "items %llu\n", stat->read_leaves, stat->by_leaf, 
+			  "item %llu\n", stat->read_leaves, stat->by_leaf, 
 			  stat->by_item_leaves);
 
 	time_str = ctime(&am->stat.time);
@@ -89,11 +66,7 @@ static void repair_add_missing_update(repair_am_t *am) {
 	time_str = ctime(&am->stat.time);
 	time_str[aal_strlen(time_str) - 1] = '\0';
 	aal_stream_format(&stream, time_str);
-	
-	am->progress->state = PROGRESS_STAT;
-	am->progress->text = (char *)stream.entity;
-	am->progress_handler(am->progress);
-	
+	aal_mess(stream.entity);
 	aal_stream_fini(&stream);
 }
 
@@ -110,9 +83,8 @@ static errno_t repair_am_node_prepare(repair_am_t *am, reiser4_node_t *node) {
 
 	for (place.pos.item = 0; place.pos.item < count; place.pos.item++) {
 		if ((res = reiser4_place_fetch(&place))) {
-			aal_error("Node (%llu), item (%u): failed to "
-				  "open the item.",node_blocknr(node),
-				  place.pos.item);
+			aal_error("Node (%llu), item (%u): failed to open the "
+				  "item.",node->block->nr, place.pos.item);
 
 			return res;
 		}
@@ -153,7 +125,7 @@ static errno_t repair_am_blk_used(repair_am_t *am, blk_t blk) {
 }
 
 typedef struct stat_bitmap {
-	uint64_t read, by_node, by_item;
+	uint64_t read, by_node, by_item, empty;
 } stat_bitmap_t;
 
 static errno_t repair_am_nodes_insert(repair_am_t *am, 
@@ -161,6 +133,7 @@ static errno_t repair_am_nodes_insert(repair_am_t *am,
 				      stat_bitmap_t *stat)
 {
 	reiser4_node_t *node;
+	uint64_t total;
 	errno_t res;
 	blk_t blk;
 	
@@ -168,6 +141,8 @@ static errno_t repair_am_nodes_insert(repair_am_t *am,
 	aal_assert("vpf-1283", bitmap != NULL);
 	aal_assert("vpf-1284", stat != NULL);
 	
+	total = aux_bitmap_marked(bitmap);
+	
 	blk = 0;
 	/* Try to insert the whole twig/leaf at once. If it can be 
 	   inserted only after splitting the node found by lookup 
@@ -176,12 +151,11 @@ static errno_t repair_am_nodes_insert(repair_am_t *am,
 	while ((blk = aux_bitmap_find_marked(bitmap, blk)) != INVAL_BLK) {
 		node = reiser4_node_open(am->repair->fs->tree, blk);
 		stat->read++;
-
-		if (am->progress_handler)
-			am->progress_handler(am->progress);
+		aal_gauge_set_value(am->gauge, stat->read * 100 / total);
+		aal_gauge_touch(am->gauge);
 
 		if (node == NULL) {
-			aal_fatal("Add Missing pass failed to "
+			aal_error("Add Missing pass failed to "
 				  "open the node (%llu)", blk);
 
 			return -EINVAL;
@@ -192,9 +166,10 @@ static errno_t repair_am_nodes_insert(repair_am_t *am,
 			goto error_close_node;
 
 		if (reiser4_node_items(node) == 0) {
-			aux_bitmap_clear(bitmap, node_blocknr(node));
-			repair_am_blk_free(am, node_blocknr(node));
+			aux_bitmap_clear(bitmap, node->block->nr);
+			repair_am_blk_free(am, node->block->nr);
 			reiser4_node_close(node);
+			stat->empty++;
 			blk++;
 			continue;
 		}
@@ -202,22 +177,22 @@ static errno_t repair_am_nodes_insert(repair_am_t *am,
 		res = repair_tree_attach_node(am->repair->fs->tree, node);
 		
 		if (res < 0 && res != -ESTRUCT) {
-			aal_bug("vpf-3068", "Add missing pass failed to attach "
-				"the node (%llu) to the tree.", blk);
+			aal_error("Add missing pass failed to attach "
+				  "the node (%llu) to the tree.", blk);
 
 			goto error_close_node;
 		} else if (res == 0) {
 			/* Has been inserted. */
-			aux_bitmap_clear(bitmap, node_blocknr(node));
-			repair_am_blk_used(am, node_blocknr(node));
+			aux_bitmap_clear(bitmap, node->block->nr);
+			repair_am_blk_used(am, node->block->nr);
 
 			stat->by_node++;
 
-			res = reiser4_node_trav(node, callback_layout, am);
+			res = reiser4_node_trav(node, cb_layout, am);
 			if (res) goto error_close_node;
 
 		} else {
-			/* uninsertable case - insert by items later. */
+			/* uninsertable case - insert by item later. */
 			reiser4_node_fini(node);
 		}
 		
@@ -237,6 +212,7 @@ static errno_t repair_am_items_insert(repair_am_t *am,
 {
 	uint32_t count;
 	reiser4_node_t *node;
+	uint64_t total;
 	errno_t res;
 	blk_t blk;
 
@@ -244,6 +220,7 @@ static errno_t repair_am_items_insert(repair_am_t *am,
 	aal_assert("vpf-1286", bitmap != NULL);
 	aal_assert("vpf-1287", stat != NULL);
 	
+	total = aux_bitmap_marked(bitmap);
 	blk = 0;
 	/* Insert extents from the twigs/all items from leaves which 
 	   are not in the tree yet item-by-item into the tree, overwrite
@@ -256,12 +233,9 @@ static errno_t repair_am_items_insert(repair_am_t *am,
 		aal_assert("vpf-897", !aux_bitmap_test(am->bm_used, blk));
 
 		node = reiser4_node_open(am->repair->fs->tree, blk);
-
-		if (am->progress_handler)
-			am->progress_handler(am->progress);
-
+		
 		if (node == NULL) {
-			aal_fatal("Add Missing pass failed to "
+			aal_error("Add Missing pass failed to "
 				  "open the node (%llu)", blk);
 			return -EINVAL;
 		}
@@ -270,6 +244,8 @@ static errno_t repair_am_items_insert(repair_am_t *am,
 		place.node = node;
 
 		stat->by_item++;
+		aal_gauge_set_value(am->gauge, stat->by_item * 100 / total);
+		aal_gauge_touch(am->gauge);
 
 		for (pos->item = 0; pos->item < count; pos->item++) {
 			pos->unit = MAX_UINT32;
@@ -283,13 +259,20 @@ static errno_t repair_am_items_insert(repair_am_t *am,
 			}
 
 			res = repair_tree_insert(am->repair->fs->tree, &place,
-						 callback_item_mark_region, am);
+						 cb_item_mark_region, am);
 			
-			if (res) goto error_close_node;
+			if (res < 0) 
+				goto error_close_node;
+			
+			if (res && place.plug->id.group == STAT_ITEM) {
+				/* If insertion cannot be performed for the 
+				   statdata item, descement file counter. */
+				(*am->stat.files)--;
+			}
 		}
 
-		aux_bitmap_clear(bitmap, node_blocknr(node));
-		repair_am_blk_free(am, node_blocknr(node));
+		aux_bitmap_clear(bitmap, node->block->nr);
+		repair_am_blk_free(am, node->block->nr);
 		reiser4_node_close(node);
 		blk++;
 	}
@@ -304,7 +287,6 @@ static errno_t repair_am_items_insert(repair_am_t *am,
 /* The pass inself, adds all the data which are not in the tree yet and which 
    were found on the partition during the previous passes. */
 errno_t repair_add_missing(repair_am_t *am) {
-	repair_progress_t progress;
 	aux_bitmap_t *bitmap;
 	stat_bitmap_t stat;
 	uint32_t bnum;
@@ -316,60 +298,55 @@ errno_t repair_add_missing(repair_am_t *am) {
 	aal_assert("vpf-848", am->bm_twig != NULL);
 	aal_assert("vpf-849", am->bm_leaf != NULL);
 	
-	am->progress = &progress;
-	
-	repair_add_missing_setup(am);
-	
+	aal_mess("INSERTING UNCONNECTED NODES");
+	am->gauge = aal_gauge_create(aux_gauge_handlers[GT_PROGRESS], 
+				     NULL, NULL, 500, NULL);
+	time(&am->stat.time);
 	
 	/* 2 loops - 1 for twigs, another for leaves. */
 	for (bnum = 0; bnum < 2; bnum++) {
 		if (bnum) {
 			bitmap = am->bm_leaf;
-			am->progress->text = "Inserting unconnected leaves: ";
-			am->progress->u.rate.total = 
-				aux_bitmap_marked(am->bm_leaf);
+			aal_gauge_rename(am->gauge, "3. Leaves: ");
 		} else {
 			bitmap = am->bm_twig;
-			am->progress->text = "Inserting unconnected twigs: ";
-			am->progress->u.rate.total = 
-				aux_bitmap_marked(am->bm_twig);
+			aal_gauge_rename(am->gauge, "1. Twigs: ");
 		}
 		
 		/* Debugging of item coping. */
-		if (am->repair->debug_flag) 
+		if (am->repair->flags & (1 << REPAIR_DEBUG)) 
 			goto debug;
 		
-		am->progress->u.rate.done = 0;
-		am->progress->state = PROGRESS_UPDATE;
-		
 		aal_memset(&stat, 0, sizeof(stat));
 		
+		aal_gauge_set_value(am->gauge, 0);
+		aal_gauge_touch(am->gauge);
+		
 		if ((res = repair_am_nodes_insert(am, bitmap, &stat)))
 			goto error;
 		
 		if (bnum) {
-			am->stat.read_twigs = stat.read;
-			am->stat.by_twig = stat.by_node;
-		} else {
 			am->stat.read_leaves = stat.read;
 			am->stat.by_leaf = stat.by_node;
-		}
-	debug:
-		am->progress->u.rate.done = 0;
-		if (bnum) {
-			am->progress->text = "Inserting unconnected twigs "
-				"item-by-item: ";
-			am->progress->u.rate.total = 
-				aux_bitmap_marked(am->bm_leaf);
 		} else {
-			am->progress->text = "Inserting unconnected leaves "
-				"item-by-item: ";
-			am->progress->u.rate.total = 
-				aux_bitmap_marked(am->bm_twig);
+			am->stat.read_twigs = stat.read;
+			am->stat.by_twig = stat.by_node;
+			am->stat.empty = stat.empty;
+		}
+	
+		aal_gauge_done(am->gauge);
+	debug:
+		if (bnum) {
+			aal_gauge_rename(am->gauge, "4. Leaves by item: ");
+		} else {
+			aal_gauge_rename(am->gauge, "2. Twigs by item: ");
 		} 
 		
 		aal_memset(&stat, 0, sizeof(stat));
 		
+		aal_gauge_set_value(am->gauge, 0);
+		aal_gauge_touch(am->gauge);
+
 		if ((res = repair_am_items_insert(am, bitmap, &stat)))
 			goto error;
 
@@ -377,19 +354,19 @@ errno_t repair_add_missing(repair_am_t *am) {
 			am->stat.by_item_leaves = stat.by_item;
 		else
 			am->stat.by_item_twigs = stat.by_item;
+		
+		aal_gauge_done(am->gauge);
 	}
 	
-	am->progress->state = PROGRESS_END;
-	if (am->progress_handler)
-		am->progress_handler(am->progress);
-
+	aal_gauge_free(am->gauge);
 	repair_add_missing_update(am);
 	reiser4_fs_sync(am->repair->fs);
 	
 	return 0;
 
  error:
-	
+	aal_gauge_done(am->gauge);
+	aal_gauge_free(am->gauge);
 	reiser4_fs_sync(am->repair->fs);
 	return res;
 }
