@@ -12,8 +12,8 @@ errno_t repair_alloc_check_struct(reiser4_alloc_t *alloc, uint8_t mode) {
 			 check_struct, alloc->ent, mode);
 }
 
-errno_t repair_alloc_layout_bad(reiser4_alloc_t *alloc, region_func_t func, 
-				void *data) 
+errno_t repair_alloc_layout_bad(reiser4_alloc_t *alloc, 
+				region_func_t func, void *data)
 {
 	aal_assert("vpf-1322", alloc != NULL);
 	
@@ -31,9 +31,14 @@ void repair_alloc_print(reiser4_alloc_t *alloc, aal_stream_t *stream) {
 
 /* Fetches block allocator data to @stream. */
 errno_t repair_alloc_pack(reiser4_alloc_t *alloc, aal_stream_t *stream) {
+	rid_t pid;
+	
 	aal_assert("umka-2614", alloc != NULL);
 	aal_assert("umka-2615", stream != NULL);
 
+	pid = alloc->ent->plug->id.id;
+	aal_stream_write(stream, &pid, sizeof(pid));
+
 	return plug_call(alloc->ent->plug->o.alloc_ops,
 			 pack, alloc->ent, stream);
 }
@@ -42,7 +47,7 @@ errno_t repair_alloc_pack(reiser4_alloc_t *alloc, aal_stream_t *stream) {
 reiser4_alloc_t *repair_alloc_unpack(reiser4_fs_t *fs, aal_stream_t *stream) {
 	reiser4_alloc_t *alloc;
 	reiser4_plug_t *plug;
-	fs_desc_t desc;
+	uint32_t blksize;
 	uint32_t read;
 	rid_t pid;
 	
@@ -67,14 +72,11 @@ reiser4_alloc_t *repair_alloc_unpack(reiser4_fs_t *fs, aal_stream_t *stream) {
 		return NULL;
 
 	alloc->fs = fs;
-	alloc->fs->alloc = alloc;
-
-	desc.device = fs->device;
-	desc.blksize = reiser4_master_get_blksize(fs->master);
+	blksize = reiser4_master_get_blksize(fs->master);
 	
 	/* Query the block allocator plugin for creating allocator entity */
 	if (!(alloc->ent = plug_call(plug->o.alloc_ops, unpack,
-				     &desc, stream)))
+				     fs->device, blksize, stream)))
 	{
 		aal_error("Can't unpack block allocator.");
 		goto error_free_alloc;
