@@ -104,7 +104,7 @@ errno_t alloc40_layout(generic_entity_t *entity,
 	for (blk = start; blk < start + alloc->bitmap->total;
 	     blk = ((blk / bpb) + 1) * bpb) 
 	{
-		res |= region_func(entity, blk, 1, data);
+		res |= region_func(blk, 1, data);
 		
 		if (res && res != -ESTRUCT)
 			return res;
@@ -117,9 +117,7 @@ errno_t alloc40_layout(generic_entity_t *entity,
    saves it in allocator checksums area. Actually this function is callback one
    which is called by alloc40_layout function in order to load all bitmap map
    from the device. See alloc40_open for details. */
-static errno_t cb_fetch_bitmap(void *entity, blk_t start,
-			       count_t width, void *data)
-{
+static errno_t cb_fetch_bitmap(blk_t start, count_t width, void *data) {
 	errno_t res;
 	uint64_t offset;
 	alloc40_t *alloc;
@@ -127,7 +125,7 @@ static errno_t cb_fetch_bitmap(void *entity, blk_t start,
 	char *current, *map;
 	uint32_t size, chunk, free;
     
-	alloc = (alloc40_t *)entity;
+	alloc = (alloc40_t *)data;
 
 	if ((res = aal_block_init(&block, alloc->device,
 				  alloc->blksize, start)))
@@ -166,13 +164,16 @@ static errno_t cb_fetch_bitmap(void *entity, blk_t start,
 /* Initializing block allocator instance and loads bitmap into it from the
    passed @device. This functions is implementation of alloc_ops.open() plugin
    method. */
-static generic_entity_t *alloc40_open(fs_desc_t *desc, uint64_t blocks) {
+static generic_entity_t *alloc40_open(aal_device_t *device, 
+				      uint32_t blksize, 
+				      uint64_t blocks) 
+{
 	alloc40_t *alloc;
 	uint32_t crcsize;
 	uint32_t mapsize;
     
 	aal_assert("umka-1682", blocks > 0);
-	aal_assert("umka-364", desc != NULL);
+	aal_assert("umka-364", device != NULL);
 
 	if (!(alloc = aal_calloc(sizeof(*alloc), 0)))
 		return NULL;
@@ -180,7 +181,7 @@ static generic_entity_t *alloc40_open(fs_desc_t *desc, uint64_t blocks) {
 	/* Creating bitmap with passed @len. Value @len is the number of blocks
 	   filesystem lies in. In other words it is the filesystem size. This
 	   value is the same as partition size sometimes. */
-	mapsize = desc->blksize - CRC_SIZE;
+	mapsize = blksize - CRC_SIZE;
     
 	if (!(alloc->bitmap = aux_bitmap_create(blocks)))
 		goto error_free_alloc;
@@ -188,8 +189,8 @@ static generic_entity_t *alloc40_open(fs_desc_t *desc, uint64_t blocks) {
 	/* Initializing alloc instance. */
 	alloc->state = 0;
 	alloc->plug = &alloc40_plug;
-	alloc->device = desc->device;
-	alloc->blksize = desc->blksize;
+	alloc->device = device;
+	alloc->blksize = blksize;
 
 	/* Calulating crc array size */
 	crcsize = ((alloc->bitmap->size + mapsize - 1) /
@@ -221,18 +222,21 @@ static generic_entity_t *alloc40_open(fs_desc_t *desc, uint64_t blocks) {
    caller (block allocator in libreiser4). This function does almost the same as
    alloc40_open(). The difference is that it does not load bitmap from the
    passed device. */
-static generic_entity_t *alloc40_create(fs_desc_t *desc, uint64_t blocks) {
+static generic_entity_t *alloc40_create(aal_device_t *device, 
+					uint32_t blksize, 
+					uint64_t blocks) 
+{
 	alloc40_t *alloc;
 	uint32_t mapsize;
 	uint32_t crcsize;
 
-	aal_assert("umka-365", desc != NULL);
+	aal_assert("umka-365", device != NULL);
 	aal_assert("umka-1683", blocks > 0);
 
 	if (!(alloc = aal_calloc(sizeof(*alloc), 0)))
 		return NULL;
 
-	mapsize = desc->blksize - CRC_SIZE;
+	mapsize = blksize - CRC_SIZE;
     
 	if (!(alloc->bitmap = aux_bitmap_create(blocks)))
 		goto error_free_alloc;
@@ -244,8 +248,8 @@ static generic_entity_t *alloc40_create(fs_desc_t *desc, uint64_t blocks) {
 
 	/* Initializing alloc instance. */
 	alloc->plug = &alloc40_plug;
-	alloc->device = desc->device;
-	alloc->blksize = desc->blksize;
+	alloc->device = device;
+	alloc->blksize = blksize;
 	alloc->state = (1 << ENTITY_DIRTY);
     
 	return (generic_entity_t *)alloc;
@@ -293,16 +297,14 @@ static errno_t alloc40_extract(generic_entity_t *entity, void *data) {
 }
 
 /* Callback for saving one bitmap block onto device */
-static errno_t cb_sync_bitmap(void *entity, blk_t start,
-			      count_t width, void *data)
-{
+static errno_t cb_sync_bitmap(blk_t start, count_t width, void *data) {
 	errno_t res;
 	alloc40_t *alloc;
 	aal_block_t block;
 	char *current, *map; 
 	uint32_t size, adler, chunk;
 	
-	alloc = (alloc40_t *)entity;
+	alloc = (alloc40_t *)data;
 	
 	/* Allocating new block and filling it by 0xff bytes (all bits are
 	   turned on). This is needed in order to make the rest of last block
@@ -512,19 +514,22 @@ typedef void (*inval_func_t) (blk_t, uint32_t, uint32_t);
 
 /* Callback function for checking one bitmap block on validness. Here we just
    calculate actual checksum and compare it with loaded one. */
-errno_t cb_valid_block(void *entity, blk_t start, count_t width, void *data) {
-	uint32_t chunk;
-	uint64_t offset;
+errno_t cb_valid_block(blk_t start, count_t width, void *data) {
+	inval_func_t inval_func;
 	alloc40_t *alloc;
 	errno_t res;
-
-	uint32_t size, free;
-	char *current, *map;
-	inval_func_t inval_func;
-	uint32_t ladler, cadler;
-
-	alloc = (alloc40_t *)entity;
-	inval_func = (inval_func_t)data;
+	
+	uint64_t offset;
+	uint32_t ladler;
+	uint32_t cadler;
+	uint32_t chunk;
+	uint32_t size;
+	uint32_t free;	
+	char *current;
+	char *map;
+	
+	alloc = (alloc40_t *)data;
+	inval_func = (inval_func_t)alloc->data;
 	
 	size = alloc->blksize - CRC_SIZE;
 	map = alloc->bitmap->map;
@@ -572,8 +577,8 @@ errno_t alloc40_valid(generic_entity_t *entity) {
 
 	/* Calling layout function for traversing all the bitmap blocks with
 	   checking callback function. */
-	return alloc40_layout((generic_entity_t *)alloc,
-			      cb_valid_block, cb_inval_warn);
+	alloc->data = cb_inval_warn;
+	return alloc40_layout(entity, cb_valid_block, alloc);
 }
 
 /* Call @func for all blocks which belong to the same bitmap block as passed
@@ -604,7 +609,7 @@ errno_t alloc40_region(generic_entity_t *entity, blk_t blk,
 	
 	/* Loop though the all blocks one bitmap block describes and calling
 	   passed @region_func for each of them. */   
-	return region_func(entity, start, size, data);
+	return region_func(start, size, data);
 }
 
 static reiser4_alloc_ops_t alloc40_ops = {
