@@ -3,6 +3,7 @@
    
    ext_t.c -- 1 symbol extention fibration code. */
 
+#ifdef ENABLE_EXT_1_FIBRE
 #include <reiser4/plugin.h>
 
 static uint8_t fibre_ext_1_build(char *name, uint32_t len) {
@@ -21,9 +22,9 @@ static reiser4_fibre_ops_t fibre_ext_1_ops = {
 static reiser4_plug_t fibre_ext_1_plug = {
 	.cl    = class_init,
 	.id    = {FIBRE_EXT_1_ID, 0, FIBRE_PLUG_TYPE},
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.label = "ext_1_fibre",
-	.desc  = "1-symbol extention fibration plugin for reiser4, ver. " VERSION,
+	.desc  = "1-symbol extention fibration plugin for reiser4. ",
 #endif
 	.o = {
 		.fibre_ops = &fibre_ext_1_ops
@@ -35,3 +36,4 @@ static reiser4_plug_t *fibre_ext_1_start(reiser4_core_t *c) {
 }
 
 plug_register(fibre_ext_1, fibre_ext_1_start, NULL);
+#endif
