@@ -17,9 +17,9 @@ static reiser4_fibre_ops_t fibre_lexic_ops = {
 static reiser4_plug_t fibre_lexic_plug = {
 	.cl    = class_init,
 	.id    = {FIBRE_LEXIC_ID, 0, FIBRE_PLUG_TYPE},
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.label = "lexic_fibre",
-	.desc  = "Lexicographic fibration plugin for reiser4, ver. " VERSION,
+	.desc  = "Lexicographic fibration plugin for reiser4. ",
 #endif
 	.o = {
 		.fibre_ops = &fibre_lexic_ops
