@@ -6,7 +6,7 @@
 #ifdef ENABLE_R5_HASH
 #include <reiser4/plugin.h>
 
-uint64_t r5_hash_build(char *name, uint32_t len) {
+uint64_t r5_hash_build(unsigned char *name, uint32_t len) {
 	uint32_t i;
 	uint64_t a = 0;
 	
@@ -26,9 +26,9 @@ reiser4_hash_ops_t r5_hash_ops = {
 static reiser4_plug_t r5_hash_plug = {
 	.cl    = class_init,
 	.id    = {HASH_R5_ID, 0, HASH_PLUG_TYPE},
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.label = "r5_hash",
-	.desc  = "R5 hash for reiser4, ver. " VERSION,
+	.desc  = "R5 hash for reiser4. ",
 #endif
 	.o = {
 		.hash_ops = &r5_hash_ops
