@@ -6,20 +6,21 @@
 #ifndef CDE40_REPAIR_H
 #define CDE40_REPAIR_H
 
+#ifndef ENABLE_MINIMAL
 #include <aal/libaal.h>
 #include <reiser4/plugin.h>
 
-extern errno_t cde40_merge(reiser4_place_t *place,
-			   trans_hint_t *hint);
+extern errno_t cde40_prep_insert_raw(reiser4_place_t *place,
+				     trans_hint_t *hint);
+
+extern errno_t cde40_insert_raw(reiser4_place_t *place,
+				trans_hint_t *hint);
 
 extern errno_t cde40_check_struct(reiser4_place_t *place,
-				  uint8_t mode);
-
-extern errno_t cde40_prep_merge(reiser4_place_t *place,
-				trans_hint_t *hint);
+				  repair_hint_t *hint);
 
 extern void cde40_print(reiser4_place_t *place,
 			aal_stream_t *stream,
 			uint16_t options);
-
+#endif
 #endif
