@@ -41,7 +41,7 @@ uint64_t extent40_offset(reiser4_place_t *place, uint32_t pos) {
 	for (i = 0; i < pos; i++, extent++)
 		blocks += et40_get_width(extent);
     
-	return blocks * extent40_blksize(place);
+	return blocks * place_blksize(place);
 }
 
 /* Builds the key of the unit at @pos and stores it inside passed @key
@@ -78,7 +78,7 @@ static uint64_t extent40_bytes(reiser4_place_t *place) {
 			blocks += et40_get_width(extent);
 	}
     
-	return (blocks * extent40_blksize(place));
+	return (blocks * place_blksize(place));
 
 }
 
@@ -91,8 +91,7 @@ uint32_t extent40_unit(reiser4_place_t *place, uint64_t offset) {
         extent = extent40_body(place);
                                                                                          
         for (i = 0; i < extent40_units(place); i++, extent++) {
-                width += et40_get_width(extent) *
-			extent40_blksize(place);
+                width += et40_get_width(extent) * place_blksize(place);
 
                 if (offset < width)
                         return i;
@@ -138,6 +137,14 @@ static errno_t extent40_remove_units(reiser4_place_t *place, trans_hint_t *hint)
 			start = et40_get_start(extent);
 			width = et40_get_width(extent);
 
+			if (start != EXTENT_HOLE_UNIT) {
+				if ((res = extent40_core->tree_ops.inc_free(
+					place->node->tree, width)))
+				{
+					return res;
+				}
+			}
+
 			if (start == EXTENT_UNALLOC_UNIT || 
 			    start == EXTENT_HOLE_UNIT)
 				continue;
@@ -175,7 +182,9 @@ static errno_t extent40_remove_units(reiser4_place_t *place, trans_hint_t *hint)
 static int64_t extent40_trunc_units(reiser4_place_t *place,
 				    trans_hint_t *hint)
 {
-	uint32_t size;
+	int32_t size;
+	uint32_t epos;
+	uint32_t skip;
 	uint64_t count;
 	uint64_t esize;
 	uint64_t offset;
@@ -195,41 +204,66 @@ static int64_t extent40_trunc_units(reiser4_place_t *place,
 	if (place->pos.unit == MAX_UINT32)
 		place->pos.unit = 0;
 
-	/* Get the amount of bytes from the current unit through the end of 
-	   the item. */
-	esize = extent40_size(place) - 
-		extent40_offset(place, place->pos.unit);
+	blksize = place_blksize(place);
+	extent = extent40_body(place) + place->pos.unit;
+	offset = plug_call(hint->offset.plug->o.key_ops, 
+			   get_offset, &hint->offset);
 
-	if ((count = hint->count) > esize)
-		count = esize;
+	/* Get the amount of bytes that cannot be cut at the start. */
+	skip = (blksize - offset % blksize);
+	if (skip == blksize)
+		skip = 0;
+
+	/* Set the start key offset to the block border. */
+	offset += skip;
+	key = hint->offset;
+	plug_call(key.plug->o.key_ops, set_offset, &key, offset);
+
+	/* Get the amount of bytes from @offset through the end of the item. */
+	offset -= plug_call(hint->offset.plug->o.key_ops, 
+			   get_offset, &place->key);
+	esize = extent40_size(place) - offset;
+
+	aal_assert("vpf-1759", offset % blksize == 0);
 	
-	extent = extent40_body(place) +
-		place->pos.unit;
-
-	blksize = extent40_blksize(place);
+	/* Get number of blocks to be skipped within the unit. */
+	epos = (offset - extent40_offset(place, place->pos.unit)) / blksize;
+	
+	if (hint->count < skip)
+		return hint->count;
+	
+	if ((count = hint->count - skip) > esize)
+		count = esize;
 
 	/* Loop though the units until @count of bytes are truncated. We do not
 	   increase @place->po.unit, because it will be the same for all ticks
 	   of the loop, as units will be removed fully or partially and this
 	   means, that @count is over. */
 	for (size = count; size > 0; ) {
-		uint32_t i;
+		uint32_t remove;
 		uint32_t start;
 		uint32_t width;
-		uint32_t remove;
+		uint32_t eskip;
+		errno_t res;
+		uint32_t i;
 
 		width = et40_get_width(extent);
+		eskip = size == (int64_t)count ? epos : 0;
 
+		aal_assert("vpf-1760", eskip < width);
+		
 		/* Calculating what is to be cut out. */
-		if ((remove = size / blksize) > width) {
-			remove = width;
-		} else if (remove == 0) {
+		if ((remove = size / blksize) > width - eskip)
+			remove = width - eskip;
+		
+		/* No one block needs to be removed anymore. */
+		if (remove == 0)
 			break;
-		}
-			
-		if (extent40_fetch_key(place, &key))
-			return -EIO;
-	
+		
+		aal_assert("vpf-1761", eskip == 0 || remove == width - eskip);
+		aal_assert("vpf-1762", place->pos.unit == 0 || 
+			   remove == width - eskip);
+		
 		/* Removing unit data from the cache */
 		for (i = 0; i < remove; i++) {
 			aal_hash_table_remove(hint->blocks, &key);
@@ -243,21 +277,31 @@ static int64_t extent40_trunc_units(reiser4_place_t *place,
 		
 		start = et40_get_start(extent);
 		
-		/* Calling region remove notification function. */
-		if (start != EXTENT_HOLE_UNIT && start != EXTENT_UNALLOC_UNIT) {
-			errno_t res;
-			
-			if ((res = hint->region_func(start, remove, 
-						     hint->data)))
+		if (start != EXTENT_HOLE_UNIT) {
+			if ((res = extent40_core->tree_ops.inc_free(
+				place->node->tree, remove)))
 			{
 				return res;
 			}
 		}
 
-		hint->bytes += remove * blksize;
+		/* Calling region remove notification function. */
+		if (start != EXTENT_HOLE_UNIT && start != EXTENT_UNALLOC_UNIT) {
+			if ((res = hint->region_func(start + eskip, 
+						     remove, hint->data)))
+			{
+				return res;
+			}
+		}
+
+		if (start != EXTENT_HOLE_UNIT)
+			hint->bytes += remove * blksize;
 		
 		/* Check if we remove whole unit. */
-		if (remove < width) {
+		if (eskip) {
+			et40_dec_width(extent, remove);
+			hint->bytes += remove * blksize;
+		} else if (remove < width) {
 			if (start != EXTENT_HOLE_UNIT &&
 			    start != EXTENT_UNALLOC_UNIT)
 			{
@@ -287,17 +331,15 @@ static int64_t extent40_trunc_units(reiser4_place_t *place,
 	}
 
 	/* Updating key if it makes sense. */
-	count = count / blksize * blksize;
-	
-	if (place->pos.unit == 0 && count) {
+	if (place->pos.unit == 0 && epos == 0 && count) {
 		offset = plug_call(place->key.plug->o.key_ops,
 				   get_offset, &place->key);
 		
 		plug_call(place->key.plug->o.key_ops, set_offset,
-			  &place->key, offset + count);
+			  &place->key, offset + count / blksize * blksize);
 	}
 	
-	return count;
+	return count + skip;
 }
 
 /* Builds maximal real key in use for specified @place */
@@ -345,8 +387,7 @@ lookup_t extent40_lookup(reiser4_place_t *place,
 			   get_offset, &place->key);
 	
 	for (i = 0; i < units; i++, extent++) {
-		offset += (et40_get_width(extent) *
-			extent40_blksize(place));
+		offset += (et40_get_width(extent) * place_blksize(place));
 
 		if (offset > wanted) {
 			place->pos.unit = i;
@@ -386,7 +427,7 @@ static int64_t extent40_read_units(reiser4_place_t *place,
 		place->pos.unit = 0;
 
 	extent40_fetch_key(place, &key);
-	blksize = extent40_blksize(place);
+	blksize = place_blksize(place);
 
 	/* Initializing read offset */
 	read_offset = plug_call(hint->offset.plug->o.key_ops,
@@ -516,7 +557,7 @@ static int64_t extent40_read_units(reiser4_place_t *place, trans_hint_t *hint) {
 	
 	extent40_fetch_key(place, &key);
 	
-	blksize = extent40_blksize(place);
+	blksize = place_blksize(place);
 	secsize = extent40_secsize(place);
 
 	read_offset = plug_call(hint->offset.plug->o.key_ops,
@@ -737,6 +778,9 @@ static int64_t extent40_insert_units(reiser4_place_t *place,
 	aal_assert("umka-2429", hint != NULL);
 	aal_assert("umka-2428", place != NULL);
 
+	if (place->pos.unit == MAX_UINT32)
+		place->pos.unit = 0;
+
 	/* Expanding extent item at @place */
 	extent40_expand(place, place->pos.unit,
 			hint->count);
@@ -759,14 +803,14 @@ static int64_t extent40_alloc_block(reiser4_place_t *place,
 	errno_t res;
 
 	device = extent40_device(place);
-	blksize = extent40_blksize(place);
+	blksize = place_blksize(place);
 
 	/* Get offset aligned to the blksize. */
 	offset = (ins_offset - (ins_offset & (blksize - 1)));
 	count = (ins_offset + count + blksize - 1) / blksize * blksize - offset;
 
 	/* Prepare the key of the new allocated block. */
-	plug_call(place->key.plug->o.key_ops, assign, &key, &place->key);
+	aal_memcpy(&key, &place->key, sizeof(key));
 	plug_call(key.plug->o.key_ops, set_offset, &key, offset);
 
 	bytes = count;
@@ -808,7 +852,7 @@ static aal_block_t *extent40_load_block(reiser4_place_t *place,
 	uint32_t blksize;
 	
 	device = extent40_device(place);
-	blksize = extent40_blksize(place);
+	blksize = place_blksize(place);
 
 	/* Obtain the block from the block hash table. */
 	if ((block = aal_hash_table_lookup(blocks, key)))
@@ -872,8 +916,7 @@ static errno_t extent40_prep_write(reiser4_place_t *place, trans_hint_t *hint) {
 
 	/* Set maxkey as the start key of the operation. The amount of bytes 
 	   handled will be added later. */
-	plug_call(hint->offset.plug->o.key_ops,
-		  assign, &hint->maxkey, &hint->offset);
+	aal_memcpy(&hint->maxkey, &hint->offset, sizeof(hint->maxkey));
 
 	if (place->pos.unit == MAX_UINT32 || unit_pos == units) {
 		/* Inserting a new item or appending units. */
@@ -913,7 +956,7 @@ static errno_t extent40_prep_write(reiser4_place_t *place, trans_hint_t *hint) {
 		/* Writing inside item. */
 
 		extent40_fetch_key(place, &key);
-		blksize = extent40_blksize(place);
+		blksize = place_blksize(place);
 
 		/* Getting maximal real key, it limits the amount of 
 		   data being written at once. */
@@ -1022,7 +1065,7 @@ static int64_t extent40_write_units(reiser4_place_t *place, trans_hint_t *hint)
 	unit_pos = place->pos.unit;
 	units = extent40_units(place);
 	device = extent40_device(place);
-	blksize = extent40_blksize(place);
+	blksize = place_blksize(place);
 
 	/* Get the offset range being handled: ins_off, max_off. */
 	ins_off = plug_call(hint->offset.plug->o.key_ops,
@@ -1278,6 +1321,7 @@ static errno_t extent40_prep_shift(reiser4_place_t *src_place,
 				   shift_hint_t *hint)
 {
 	int check_point;
+	uint32_t units;
 	
 	aal_assert("umka-1705", hint != NULL);
 	aal_assert("umka-1704", src_place != NULL);
@@ -1293,27 +1337,35 @@ static errno_t extent40_prep_shift(reiser4_place_t *src_place,
 		/* If we have to take into account insert point. */
 		if (hint->control & SF_UPDATE_POINT && check_point) {
 			left = hint->pos.unit * sizeof(extent40_t);
+		
+			if (hint->control & SF_HOLD_POS &&
+			    hint->control & SF_MOVE_POINT)
+			{
+				units = extent40_units(src_place);
+				if (units > hint->pos.unit)
+					left += sizeof(extent40_t);
+			}
 			
 			if (hint->units_bytes > left)
 				hint->units_bytes = left;
 
-			hint->pos.unit -= hint->units_bytes /
-				sizeof(extent40_t);
-
-			if (hint->pos.unit == 0 &&
+			/* Units to be moved. */
+			units = hint->units_bytes / sizeof(extent40_t);
+			
+			if (hint->pos.unit <= units &&
 			    (hint->control & SF_MOVE_POINT))
 			{
 				hint->result |= SF_MOVE_POINT;
 
-				if (dst_place) {
-					hint->pos.unit = dst_place->len +
-						hint->units_bytes;
-				
-					hint->pos.unit /= sizeof(extent40_t);
-				} else {
-					hint->pos.unit = hint->units_bytes /
-						sizeof(extent40_t);
-				}
+				hint->pos.unit = dst_place ? dst_place->len : 0;
+				hint->pos.unit += hint->units_bytes;
+				hint->pos.unit /= sizeof(extent40_t);
+
+				if (hint->control & SF_HOLD_POS)
+					hint->pos.unit--;
+			} else {
+				hint->pos.unit -= hint->units_bytes /
+					sizeof(extent40_t);
 			}
 		} else {
 			if (hint->units_bytes > src_place->len)
@@ -1324,28 +1376,30 @@ static errno_t extent40_prep_shift(reiser4_place_t *src_place,
 
 		/* The same check as abowe, but for right shift */
 		if (hint->control & SF_UPDATE_POINT && check_point) {
-			uint32_t units;
-
 			units = extent40_units(src_place);
 			
 			/* Check is it is possible to move something into right
 			   neighbour item. */
 			if (hint->pos.unit < units) {
-				right = (units - hint->pos.unit) *
-					sizeof(extent40_t);
-		
+				units -= hint->pos.unit;
+				units *= sizeof(extent40_t);
+				right = units;
+	
+				if (hint->control & SF_HOLD_POS &&
+				    !(hint->control & SF_MOVE_POINT))
+				{
+					right -= sizeof(extent40_t);
+				}
+
 				if (hint->units_bytes > right)
 					hint->units_bytes = right;
 
 				if ((hint->control & SF_MOVE_POINT) &&
-				    hint->units_bytes == right)
+				    hint->units_bytes == units)
 				{
 					hint->result |= SF_MOVE_POINT;
 					hint->pos.unit = 0;
-				} else {
-					hint->units_bytes = right -
-						hint->units_bytes;
-				}
+				} 
 			} else {
 				/* There is noning to move, update insert point,
 				   flags and out. */
@@ -1418,7 +1472,7 @@ static errno_t extent40_shift_units(reiser4_place_t *src_place,
 
 static item_balance_ops_t balance_ops = {
 #ifndef ENABLE_MINIMAL
-	.fuse		  = NULL,
+	.merge		  = NULL,
 	.update_key	  = NULL,
 	.mergeable	  = extent40_mergeable,
 	.prep_shift	  = extent40_prep_shift,
@@ -1468,6 +1522,7 @@ static item_debug_ops_t debug_ops = {
 #endif
 
 static item_tree_ops_t tree_ops = {
+	.init		  = NULL,
 	.down_link	  = NULL,
 #ifndef ENABLE_MINIMAL
 	.update_link	  = NULL,
