@@ -102,18 +102,15 @@ errno_t extent40_check_layout(reiser4_place_t *place,
 			result = RE_FIXABLE;
 	}
 	
-	units = extent40_join_units(place, hint->mode != RM_CHECK);
-	
-	if (units) {
-		fsck_mess("Node (%llu), item (%u): %u mergable units were "
-			  "found in the extent40 unit.%s", place_blknr(place),
-			  place->pos.item, units, hint->mode == RM_CHECK ? "" : 
-			  " Fixed.");
-		
-		if (hint->mode != RM_CHECK)
+	if (hint->mode != RM_CHECK) {
+		if ((units = extent40_join_units(place, 1))) {
+			fsck_mess("Node (%llu), item (%u): %u mergable units "
+				  "were found in the extent40 unit.%s", 
+				  place_blknr(place), place->pos.item, units, 
+				  hint->mode == RM_CHECK ? "" : " Merged.");
+			
 			hint->len += (units * sizeof(extent40_t));
-		else 
-			result |= RE_FIXABLE;
+		}
 	}
 	
 	return result;
@@ -137,7 +134,7 @@ errno_t extent40_check_struct(reiser4_place_t *place, repair_hint_t *hint) {
 	
 	/* Offset must be divisible by block size. */
 	if (plug_call(place->key.plug->o.key_ops, get_offset, &place->key) %
-	    extent40_blksize(place)) 
+	    place_blksize(place)) 
 	{
 		fsck_mess("Node (%llu), item (%u), [%s]: extent40 "
 			  "item with not valid key offset found.",
@@ -178,26 +175,23 @@ errno_t extent40_check_struct(reiser4_place_t *place, repair_hint_t *hint) {
 			res |= RE_FIXABLE;
 	}
 	
-	units = extent40_join_units(place, hint->mode != RM_CHECK);
-
-	if (units) {
-		fsck_mess("Node (%llu), item (%u): %u mergable units were "
-			  "found in the extent40 unit.%s", place_blknr(place),
-			  place->pos.item, units, hint->mode == RM_CHECK ? "" : 
-			  " Fixed.");
-		
-		if (hint->mode != RM_CHECK)
+	if (hint->mode != RM_CHECK) {
+		if ((units = extent40_join_units(place, 1))) {
+			fsck_mess("Node (%llu), item (%u): %u mergable units "
+				  "were found in the extent40 unit.%s", 
+				  place_blknr(place), place->pos.item, units, 
+				  hint->mode == RM_CHECK ? "" : " Merged.");
+			
 			hint->len += (units * sizeof(extent40_t));
-		else 
-			res |= RE_FIXABLE;
+		}
 	}
 	
 	return res;
 }
 
-static inline uint32_t extent40_head(reiser4_place_t *place, 
-				     uint32_t pos, 
-				     reiser4_key_t *key) 
+static uint32_t extent40_head(reiser4_place_t *place, 
+			      uint32_t pos, 
+			      reiser4_key_t *key) 
 {
 	uint64_t koffset, offset, doffset;
 
@@ -209,9 +203,9 @@ static inline uint32_t extent40_head(reiser4_place_t *place,
 	offset = koffset - offset;
 	doffset = extent40_offset(place, pos);
 	
-	aal_assert("vpf-1381", offset >= doffset);
+	aal_assert("vpf-1787", offset >= doffset);
 	
-	return (offset - doffset) / extent40_blksize(place);
+	return (offset - doffset) / place_blksize(place);
 }
 
 errno_t extent40_prep_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
@@ -226,8 +220,8 @@ errno_t extent40_prep_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
 
 	src = (reiser4_place_t *)hint->specific;
 
-	sextent = extent40_body(src) + src->pos.unit;
-	dextent = extent40_body(place) + place->pos.unit;
+	sextent = extent40_body(src);
+	dextent = extent40_body(place);
 	
 	/* Get the offset of the dst item key. */
 	offset = plug_call(hint->offset.plug->o.key_ops, 
@@ -237,7 +231,9 @@ errno_t extent40_prep_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
 	send = src->pos.unit - 1;
 	sunits = extent40_units(src);
 	
-	/* Get the head within the first @src unit. */
+	/* Get the head within the first @src unit. Head should be calculated 
+	   here because we may try to insert not from the beginning of the @src
+	   in any case. */
 	hint->head = extent40_head(src, src->pos.unit, &hint->offset);
 	hint->tail = 0;
 	hint->insert_flags = 0;
@@ -246,11 +242,10 @@ errno_t extent40_prep_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
 	    place->pos.unit == extent40_units(place))
 	{
 		/* The whole item to be inserted. */
-		send = extent40_units(src) - 1;
+		send = sunits - 1;
 	} else if (plug_call(place->key.plug->o.key_ops, compfull, 
 			     &hint->offset, &place->key) < 0)
 	{
-		/* The item head to be inserted. */		
 		/* Get the @src end unit and the tail within it. */
 		offset -= plug_call(hint->offset.plug->o.key_ops,
 				    get_offset, &src->key);
@@ -258,7 +253,9 @@ errno_t extent40_prep_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
 		send = extent40_unit(src, offset - 1);
 		hint->tail = et40_get_width(sextent + send) - 
 			extent40_head(src, send, &place->key);
-	} else if (!et40_get_start(dextent) && et40_get_start(sextent)) {
+	} else if (!et40_get_start(dextent + place->pos.unit) && 
+		   et40_get_start(sextent + src->pos.unit)) 
+	{
 		/* Estimate the overwrite. */
 		hint->insert_flags |= ET40_OVERWRITE;
 
@@ -272,8 +269,11 @@ errno_t extent40_prep_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
 		
 		if (send < sunits) {
 			hint->tail = (extent40_offset(src, send + 1) - offset)
-				/ extent40_blksize(src);
+				/ place_blksize(src);
 		} else {
+			/* The src item finished earlier then dst item by key 
+			   offset, then some part of the dst item will not be
+			   overwritten, set TAIL falg here. */
 			send = sunits - 1;
 			hint->tail = 0;
 			hint->insert_flags |= ET40_TAIL;
@@ -290,6 +290,9 @@ errno_t extent40_prep_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
 	hint->len = hint->count;
 	
 	if (hint->insert_flags & ET40_OVERWRITE) {
+		/* If this is an overrite, the current dst unit can be used 
+		   instead of adding another one (if there is no head in dst)
+		   and one more unit should be inserted (if there is a tail). */
 		hint->len += (hint->insert_flags & ET40_TAIL ? 1 : 0)
 			- (hint->insert_flags & ET40_HEAD ? 0 : 1);
 	}
@@ -314,13 +317,19 @@ int64_t extent40_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
 	sextent = extent40_body(src);
 	dextent = extent40_body(place);
 
-	dstart = place->pos.unit == MAX_UINT32 ? 0 : place->pos.unit;
-	sstart = src->pos.unit == MAX_UINT32 ? 0 : src->pos.unit;
-	dstart += (hint->insert_flags & ET40_HEAD ? 1 : 0);
+	if (place->pos.unit == MAX_UINT32)
+		place->pos.unit = 0;
 	
+	/* @head & @tail were calculated for the start unit, but 
+	   if ET40_HEAD flag is set, data should be inserted to 
+	   the next unit. */
+	dstart = place->pos.unit + 
+		(hint->insert_flags & ET40_HEAD ? 1 : 0);
+	
+	sstart = src->pos.unit == MAX_UINT32 ? 0 : src->pos.unit;
+
 	/* Set the maxkey of the passed operation. */
-	plug_call(src->key.plug->o.key_ops, assign, &hint->maxkey, 
-		  &hint->offset);
+	aal_memcpy(&hint->maxkey, &hint->offset, sizeof(hint->maxkey));
 	
 	if (!hint->count) {
 		/* If there is nothing to be done, skip as much as possible. */
@@ -361,17 +370,20 @@ int64_t extent40_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
 
 		return 0;
 	}
-	
-	/* Get the start key offset. */
-	offset = plug_call(hint->offset.plug->o.key_ops,
-			   get_offset, &hint->offset);
 
-	count = hint->count + (hint->insert_flags & ET40_TAIL ? 1 : 0)
-		- (hint->insert_flags & ET40_HEAD ? 0 : 1);
+	count = hint->count;
+
+	if (hint->insert_flags & ET40_OVERWRITE) {
+		count += hint->insert_flags & ET40_TAIL ? 1 : 0;
+		count -= hint->insert_flags & ET40_HEAD ? 0 : 1;
+	}
 
 	/* Set the maxkey offset correctly. */
-	offset += extent40_offset(src, src->pos.unit + hint->count);
-	offset -= hint->tail * extent40_blksize(src);
+	offset = plug_call(hint->offset.plug->o.key_ops, 
+			   get_offset, &src->key);
+
+	offset += extent40_offset(src, sstart + hint->count);
+	offset -= hint->tail * place_blksize(src);
 	plug_call(hint->offset.plug->o.key_ops, set_offset, 
 		  &hint->maxkey, offset);
 
@@ -386,19 +398,20 @@ int64_t extent40_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
 		/* Get the amount of blocks to be left in the tail. */
 		head = extent40_head(place, place->pos.unit, &hint->offset);
 	} 
-	
+
 	/* Expanding extent item at @place */
 	extent40_expand(place, dstart, count);
 
 	/* If some tail should be cut off the current dst unit, set the 
 	   correct width there. */
 	if (hint->insert_flags & ET40_TAIL) {
+		uint64_t width;
+		
 		/* Set the correct width. Start is 0 because allocated 
 		   units are not overwritten. */
-		et40_set_start(dextent + dstart + count - 1, 0);
-		et40_set_width(dextent + dstart + count - 1, 
-			       et40_get_width(dextent + place->pos.unit) 
-			       - tail);
+		et40_set_start(dextent + place->pos.unit + count, 0);
+		width = et40_get_width(dextent + place->pos.unit) - tail;
+		et40_set_width(dextent + place->pos.unit + count, width);
 
 		/* Fix the current dst unit after cutting. */
 		et40_set_width(dextent + place->pos.unit, tail);
@@ -411,9 +424,6 @@ int64_t extent40_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
 
 		/* Fix the current dst unit. */
 		et40_set_width(dextent + place->pos.unit, head);
-
-		/* Move to the next unit. */
-		dextent++;
 	}
 
 	dextent += dstart;
@@ -467,8 +477,7 @@ int64_t extent40_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
 	if (plug_call(place->key.plug->o.key_ops, compfull, 
 		      &hint->offset, &place->key) < 0)
 	{
-		plug_call(place->key.plug->o.key_ops, assign,
-			  &place->key, &hint->offset);
+		aal_memcpy(&place->key, &hint->offset, sizeof(place->key));
 	}
 		
 	place_mkdirty(place);
