@@ -114,7 +114,7 @@ static errno_t nodeptr40_remove_units(reiser4_place_t *place,
 
 static item_balance_ops_t balance_ops = {
 #ifndef ENABLE_MINIMAL
-	.fuse		  = NULL,
+	.merge		  = NULL,
 	.prep_shift	  = NULL,
 	.shift_units	  = NULL,
 	.maxreal_key	  = NULL,
@@ -166,6 +166,7 @@ static item_debug_ops_t debug_ops = {
 #endif
 
 static item_tree_ops_t tree_ops = {
+	.init		  = NULL,
 	.down_link	  = nodeptr40_down_link,
 #ifndef ENABLE_MINIMAL
 	.update_link	  = nodeptr40_update_link
