@@ -8,8 +8,6 @@
 #include "journal40.h"
 #include "journal40_repair.h"
 
-extern reiser4_plug_t journal40_plug;
-
 static uint32_t journal40_get_state(generic_entity_t *entity) {
 	aal_assert("umka-2081", entity != NULL);
 	return ((journal40_t *)entity)->state;
@@ -41,7 +39,7 @@ static errno_t journal40_layout(generic_entity_t *entity,
 	journal = (journal40_t *)entity;
 	
 	blk = JOURNAL40_BLOCKNR(journal->blksize);
-	return region_func(entity, blk, 2, data);
+	return region_func(blk, 2, data);
 }
 
 aal_device_t *journal40_device(generic_entity_t *entity) {
@@ -51,10 +49,8 @@ aal_device_t *journal40_device(generic_entity_t *entity) {
 
 /* Helper function for fetching journal footer and header in initialization
    time. */
-static errno_t cb_fetch_journal(void *entity, blk_t start,
-				count_t width, void *data)
-{
-	journal40_t *journal = (journal40_t *)entity;
+static errno_t cb_fetch_journal(blk_t start, count_t width, void *data) {
+	journal40_t *journal = (journal40_t *)data;
 
 	/* Load journal header. */
 	if (!(journal->header = aal_block_load(journal->device,
@@ -73,6 +69,8 @@ static errno_t cb_fetch_journal(void *entity, blk_t start,
 	{
 		aal_error("Can't read journal footer from block %llu. %s.",
 				    start + 1, journal->device->error);
+		
+		aal_block_free(journal->header);
 		return -EIO;
 	}
     
@@ -81,13 +79,14 @@ static errno_t cb_fetch_journal(void *entity, blk_t start,
 
 /* Open journal on passed @format entity, @start and @blocks. Uses passed @desc
    for getting device journal is working on and fs block size. */
-static generic_entity_t *journal40_open(fs_desc_t *desc, generic_entity_t *format,
-					generic_entity_t *oid, uint64_t start, 
-					uint64_t blocks)
+static generic_entity_t *journal40_open(aal_device_t *device, uint32_t blksize, 
+					generic_entity_t *format, 
+					generic_entity_t *oid, 
+					uint64_t start, uint64_t blocks)
 {
 	journal40_t *journal;
 
-	aal_assert("umka-409", desc != NULL);
+	aal_assert("umka-409", device != NULL);
 	aal_assert("umka-1692", format != NULL);
 
 	/* Initializign journal entity. */
@@ -97,9 +96,9 @@ static generic_entity_t *journal40_open(fs_desc_t *desc, generic_entity_t *forma
 	journal->state = 0;
 	journal->format = format;
 	journal->oid = oid;
-	journal->device = desc->device;
+	journal->device = device;
 	journal->plug = &journal40_plug;
-	journal->blksize = desc->blksize;
+	journal->blksize = blksize;
 
 	journal->area.len = blocks;
 	journal->area.start = start;
@@ -122,10 +121,8 @@ static generic_entity_t *journal40_open(fs_desc_t *desc, generic_entity_t *forma
 
 /* Helper function for creating empty journal header and footer. Used in journal
    create time. */
-static errno_t cb_alloc_journal(void *entity, blk_t start,
-				count_t width, void *data)
-{
-	journal40_t *journal = (journal40_t *)entity;
+static errno_t cb_alloc_journal(blk_t start, count_t width, void *data) {
+	journal40_t *journal = (journal40_t *)data;
 	
 	if (!(journal->header = aal_block_alloc(journal->device,
 						journal->blksize,
@@ -142,20 +139,26 @@ static errno_t cb_alloc_journal(void *entity, blk_t start,
 	{
 		aal_error("Can't alloc journal footer "
 			  "on block %llu.", start + 1);
+		
+		aal_block_free(journal->header);
 		return -ENOMEM;
 	}
 
+	aal_block_fill(journal->header, 0);
+	aal_block_fill(journal->footer, 0);
+	
 	return 0;
 }
 
 /* Create journal entity on passed params. Return create instance to caller. */
-static generic_entity_t *journal40_create(fs_desc_t *desc, generic_entity_t *format,
-					  generic_entity_t *oid, uint64_t start, 
-					  uint64_t blocks)
+static generic_entity_t *journal40_create(aal_device_t *device, uint32_t blksize, 
+					  generic_entity_t *format,
+					  generic_entity_t *oid, 
+					  uint64_t start, uint64_t blocks)
 {
 	journal40_t *journal;
     
-	aal_assert("umka-1057", desc != NULL);
+	aal_assert("umka-1057", device != NULL);
 	aal_assert("umka-1691", format != NULL);
 
 	/* Initializing journal entity. Making it dirty. Setting up all
@@ -167,10 +170,10 @@ static generic_entity_t *journal40_create(fs_desc_t *desc, generic_entity_t *for
 	journal->oid = oid;
 	journal->area.len = blocks;
 	journal->area.start = start;
-	journal->device = desc->device;
+	journal->device = device;
 	journal->plug = &journal40_plug;
-	journal->blksize = desc->blksize;
-	journal->state = (1 << ENTITY_DIRTY);
+	journal->blksize = blksize;
+	journal40_mkdirty(journal);
 
 	/* Create journal header and footer. */
 	if (journal40_layout((generic_entity_t *)journal,
@@ -189,10 +192,8 @@ static generic_entity_t *journal40_create(fs_desc_t *desc, generic_entity_t *for
 
 /* Helper function for save jopurnal header and footer to device journal is
    working on. */
-static errno_t cb_sync_journal(void *entity, blk_t start,
-			       count_t width, void *data)
-{
-	journal40_t *journal = (journal40_t *)entity;
+static errno_t cb_sync_journal(blk_t start, count_t width, void *data) {
+	journal40_t *journal = (journal40_t *)data;
 	
 	if (aal_block_write(journal->header)) {
 		aal_error("Can't write journal header. %s.",
@@ -218,8 +219,7 @@ static errno_t journal40_sync(generic_entity_t *entity) {
 	if ((res = journal40_layout(entity, cb_sync_journal, entity)))
 		return res;
 	
-	((journal40_t *)entity)->state &= ~(1 << ENTITY_DIRTY);
-	
+	journal40_mkclean(entity);
 	return 0;
 }
 
@@ -276,7 +276,7 @@ static errno_t journal40_update(journal40_t *journal) {
 	set_jf_used_oids(footer, get_th_used_oids(tx_header));
 	set_jf_next_oid(footer, get_th_next_oid(tx_header));
 
-	journal->state |= (1 << ENTITY_DIRTY);
+	journal40_mkdirty(journal);
 
  error_free_tx_block:
 	aal_block_free(tx_block);
@@ -601,9 +601,12 @@ static errno_t journal40_replay(generic_entity_t *entity) {
 	if ((res = journal40_update(journal)))
 		return res;
 
-	aal_mess("Reiser4 journal (%s) on %s: %llu transactions replayed "
-		 "of the total %llu blocks.", journal40_plug.label, 
-		 journal->device->name, count.tx_count, count.blk_count);
+	if (count.tx_count) {
+		aal_mess("Reiser4 journal (%s) on %s: %llu transactions "
+			 "replayed of the total %llu blocks.", 
+			 journal40_plug.label, journal->device->name, 
+			 count.tx_count, count.blk_count);
+	}
 
 	/* Invalidate the journal. */
 	journal40_invalidate(entity);
@@ -636,9 +639,11 @@ static reiser4_journal_ops_t journal40_ops = {
 	.get_state  	= journal40_get_state,
 	.check_struct	= journal40_check_struct,
 	.invalidate	= journal40_invalidate,
+	.pack		= journal40_pack,
+	.unpack		= journal40_unpack,
 };
 
-static reiser4_plug_t journal40_plug = {
+reiser4_plug_t journal40_plug = {
 	.cl    = class_init,
 	.id    = {JOURNAL_REISER40_ID, 0, JOURNAL_PLUG_TYPE},
 	.label = "journal40",
