@@ -43,6 +43,8 @@ union key_large {
 
 typedef union key_large key_large_t;
 
+extern reiser4_plug_t key_large_plug;
+
 typedef enum {
 	/* Major locality occupies higher 60 bits of the first element */
 	KEY_LARGE_LOCALITY_MASK		= 0xfffffffffffffff0ull,
@@ -120,14 +122,14 @@ extern uint64_t key_large_get_fobjectid(reiser4_key_t *key);
 static inline uint64_t kl_get_el(const key_large_t *key,
 				 key_large_field_t off)
 {
-	return LE64_TO_CPU(key->el[off]);
+	return LE64_TO_CPU(get_unaligned(key->el + off));
 }
 
 static inline void kl_set_el(key_large_t *key,
 			     key_large_field_t off,
 			     uint64_t value)
 {
-	key->el[off] = CPU_TO_LE64(value);
+	put_unaligned(CPU_TO_LE64(value), key->el + off);
 }
 
 static inline int kl_comp_el(void *k1, void *k2, int off) {
