@@ -6,8 +6,8 @@
 #ifndef DIR40_REPAIR_H
 #define DIR40_REPAIR_H
 
-#include <aal/libaal.h>
-#include <reiser4/plugin.h>
+#include "dir40.h"
+#include "plugin/object/obj40/obj40_repair.h"
 
 extern errno_t dir40_form(object_entity_t *object);
 extern object_entity_t *dir40_fake(object_info_t *info);
@@ -23,4 +23,3 @@ extern errno_t dir40_check_struct(object_entity_t *object,
 				  place_func_t place_func,
 				  void *data, uint8_t mode);
 #endif
-
