@@ -55,8 +55,7 @@ static int reg40_check_size(obj40_t *obj, uint64_t *sd_size,
 	if (reg->body_plug && reg->body_plug->id.group == EXTENT_ITEM) {
 		/* The last extent block can be not used up. */
 		if (*sd_size < counted_size &&
-		    *sd_size + STAT_PLACE(obj)->node->block->size > 
-		    counted_size)
+		    *sd_size + place_blksize(STAT_PLACE(obj)) > counted_size)
 		{
 			return 0;
 		}
@@ -129,7 +128,7 @@ static errno_t reg40_check_ikey(reg40_t *reg) {
 	offset = plug_call(reg->body.key.plug->o.key_ops, get_offset, 
 			   &reg->body.key);
 	
-	return offset % reg->body.node->block->size ? RE_FATAL : 0;
+	return offset % place_blksize(&reg->body) ? RE_FATAL : 0;
 }
 
 static errno_t reg40_next(object_entity_t *object, uint8_t mode) {
@@ -258,8 +257,7 @@ static int reg40_conv_prepare(reg40_t *reg, conv_hint_t *hint,
 			return 0;
 		
 		/* Set the start key for convertion. */
-		plug_call(reg->body.key.plug->o.key_ops, assign,
-			  &hint->offset, &reg->position);
+		aal_memcpy(&hint->offset, &reg->position, sizeof(hint->offset));
 		plug_call(reg->body.key.plug->o.key_ops, set_offset,
 			  &hint->offset, 0);
 
@@ -275,8 +273,7 @@ static int reg40_conv_prepare(reg40_t *reg, conv_hint_t *hint,
 	hint->plug = reg->body_plug;
 	
 	if (hint->offset.plug == NULL) {
-		plug_call(reg->body.key.plug->o.key_ops, assign,
-			  &hint->offset, &reg->position);
+		aal_memcpy(&hint->offset, &reg->position, sizeof(hint->offset));
 
 		hint->bytes = 0;
 	}
@@ -285,7 +282,7 @@ static int reg40_conv_prepare(reg40_t *reg, conv_hint_t *hint,
 	hint->count = maxreal + 1 - 
 		plug_call(reg->body.key.plug->o.key_ops,
 			  get_offset, &hint->offset);
-
+	
 	/* Convertion is postponed; do not bother with it for not RM_BUILD. */
 	return 1;
 }
@@ -450,10 +447,8 @@ errno_t reg40_check_struct(object_entity_t *object, place_func_t func,
 				  mode == RM_BUILD ? " Converted." : "");
 
 			if (mode == RM_BUILD) {
-				result = reg40_core->flow_ops.convert(info->tree,
-								      &conv);
-
-				if (result) return result;
+				if ((result = obj40_convert(&reg->obj, &conv)))
+					return result;
 
 				/* Evth was converted, update bytes. */
 				hint.bytes += conv.bytes;
