@@ -46,7 +46,7 @@ static errno_t spl40_stat(object_entity_t *entity, stat_hint_t *hint) {
 	return obj40_load_stat(&spl->obj, hint);
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Creates special file and returns initialized instance to the caller. */
 static object_entity_t *spl40_create(object_hint_t *hint) {
 	spl40_t *spl;
@@ -77,13 +77,13 @@ static errno_t spl40_clobber(object_entity_t *entity) {
 }
 
 /* Return number of hard links. */
-static uint32_t spl40_links(object_entity_t *entity) {
+static bool_t spl40_linked(object_entity_t *entity) {
 	spl40_t *spl;
 	
 	aal_assert("umka-2537", entity != NULL);
 
 	spl = (spl40_t *)entity;
-	return obj40_links(&spl->obj);
+	return obj40_links(&spl->obj) != 0;
 }
 
 /* Adds one more hardlink */
@@ -139,12 +139,12 @@ static void spl40_close(object_entity_t *entity) {
 }
 
 static reiser4_object_ops_t spl40_ops = {
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.create	        = spl40_create,
 	.metadata       = spl40_metadata,
 	.link           = spl40_link,
 	.unlink         = spl40_unlink,
-	.links          = spl40_links,
+	.linked         = spl40_linked,
 	.clobber        = spl40_clobber,
 	.update         = spl40_update,
 	.check_struct	= spl40_check_struct,
@@ -181,9 +181,9 @@ static reiser4_object_ops_t spl40_ops = {
 reiser4_plug_t spl40_plug = {
 	.cl    = class_init,
 	.id    = {OBJECT_SPL40_ID, SPL_OBJECT, OBJECT_PLUG_TYPE},
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.label = "spl40",
-	.desc  = "Special file plugin for reiser4, ver. " VERSION,
+	.desc  = "Special file plugin for reiser4. ",
 #endif
 	.o = {
 		.object_ops = &spl40_ops
