@@ -3,7 +3,7 @@
    
    tails.c -- tails only tail policy plugin. */
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 #include <reiser4/plugin.h>
 
 static int tails_tails(uint64_t value) {
@@ -18,7 +18,7 @@ static reiser4_plug_t tails_plug = {
 	.cl    = class_init,
 	.id    = {TAIL_ALWAYS_ID, 0, POLICY_PLUG_TYPE},
 	.label = "tails",
-	.desc  = "Tail policy \"tails only\" for reiser4, ver. " VERSION,
+	.desc  = "Tail policy \"tails only\" for reiser4. ",
 	.o = {
 		.policy_ops = &tails_ops
 	}
