@@ -7,62 +7,62 @@
 #include "sdext_lw.h"
 #include <aux/aux.h>
 
+reiser4_core_t *sdext_lw_core = NULL;
+
 /* Loads extension to passed @hint */
-static errno_t sdext_lw_open(void *body, void *hint) {
+static errno_t sdext_lw_open(stat_entity_t *stat, void *hint) {
 	sdext_lw_t *ext;
-	sdext_lw_hint_t *sdext_lw;
+	sdhint_lw_t *lwh;
     
-	aal_assert("umka-1188", body != NULL);
+	aal_assert("umka-1188", stat != NULL);
 	aal_assert("umka-1189", hint != NULL);
 
-	ext = (sdext_lw_t *)body;
-	sdext_lw = (sdext_lw_hint_t *)hint;
+	ext = (sdext_lw_t *)stat_body(stat);
+	lwh = (sdhint_lw_t *)hint;
     
-	sdext_lw->mode = sdext_lw_get_mode(ext);
-	sdext_lw->nlink = sdext_lw_get_nlink(ext);
-	sdext_lw->size = sdext_lw_get_size(ext);
+	lwh->mode = sdext_lw_get_mode(ext);
+	lwh->nlink = sdext_lw_get_nlink(ext);
+	lwh->size = sdext_lw_get_size(ext);
     
 	return 0;
 }
 
-static uint16_t sdext_lw_length(void *body) {
+static uint32_t sdext_lw_length(stat_entity_t *stat, void *hint) {
 	return sizeof(sdext_lw_t);
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Saves all extension fields from passed @hint to @body. */
-static errno_t sdext_lw_init(void *body, 
-			     void *hint) 
-{
-	sdext_lw_hint_t *sdext_lw;
-    
-	aal_assert("umka-1186", body != NULL);
+static errno_t sdext_lw_init(stat_entity_t *stat, void *hint) {
+	sdhint_lw_t *lwh;
+	sdext_lw_t *ext;
+	
+	aal_assert("umka-1186", stat != NULL);
 	aal_assert("umka-1187", hint != NULL);
 	
-	sdext_lw = (sdext_lw_hint_t *)hint;
-    
-	sdext_lw_set_mode((sdext_lw_t *)body,
-			  sdext_lw->mode);
+	lwh = (sdhint_lw_t *)hint;
+	ext = (sdext_lw_t *)stat_body(stat);
 	
-	sdext_lw_set_nlink((sdext_lw_t *)body,
-			   sdext_lw->nlink);
-	
-	sdext_lw_set_size((sdext_lw_t *)body,
-			  sdext_lw->size);
+	sdext_lw_set_mode(ext, lwh->mode);
+	sdext_lw_set_nlink(ext, lwh->nlink);
+	sdext_lw_set_size(ext, lwh->size);
 
 	return 0;
 }
 
-extern errno_t sdext_lw_check_struct(sdext_entity_t *sdext, uint8_t mode);
+extern errno_t sdext_lw_check_struct(stat_entity_t *stat, 
+				     repair_hint_t *hint);
 
-extern void sdext_lw_print(void *body, aal_stream_t *stream, uint16_t options);
+extern void sdext_lw_print(stat_entity_t *stat, 
+			   aal_stream_t *stream, 
+			   uint16_t options);
 
 #endif
 
 static reiser4_sdext_ops_t sdext_lw_ops = {
 	.open	 	= sdext_lw_open,
-		
-#ifndef ENABLE_STAND_ALONE
+	
+#ifndef ENABLE_MINIMAL
 	.init	 	= sdext_lw_init,
 	.print   	= sdext_lw_print,
 	.check_struct   = sdext_lw_check_struct,
@@ -73,9 +73,9 @@ static reiser4_sdext_ops_t sdext_lw_ops = {
 static reiser4_plug_t sdext_lw_plug = {
 	.cl    = class_init,
 	.id    = {SDEXT_LW_ID, 0, SDEXT_PLUG_TYPE},
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.label = "sdext_lw",
-	.desc  = "Light stat data extension for reiser4, ver. " VERSION,
+	.desc  = "Light stat data extension for reiser4. ",
 #endif
 	.o = {
 		.sdext_ops = &sdext_lw_ops
@@ -83,6 +83,7 @@ static reiser4_plug_t sdext_lw_plug = {
 };
 
 static reiser4_plug_t *sdext_lw_start(reiser4_core_t *c) {
+	sdext_lw_core = c;
 	return &sdext_lw_plug;
 }
 
