@@ -58,7 +58,7 @@ static errno_t fsck_ask_confirmation(fsck_parse_t *data, char *host_name) {
 		break;
 	case RM_FIX:
 		aal_mess("Will fix minor corruptions of "
-			 "the Reiser4 SuperBblock.");
+			 "the Reiser4 SuperBlock.");
 		break;
 	case RM_BUILD:
 		aal_mess("Will build the Reiser4 SuperBlock.");
@@ -140,13 +140,14 @@ static errno_t fsck_init(fsck_parse_t *data,
 		{"no-log", no_argument, NULL, 'n'},
 		{"auto", no_argument, NULL, 'a'},
 		{"force", no_argument, NULL, 'f'},
-		{"cashe", required_argument, 0, 'c'},
+		{"cache", required_argument, 0, 'c'},
 		{"override", required_argument, NULL, 'o'},
 		/* Fsck hidden options. */
 		{"passes-dump", required_argument, 0, 'U'},
 		{"backup", required_argument, 0, 'b'},
 		{"bitmap", required_argument, 0, 'B'},
 		{"unused", no_argument, NULL, 'u'},
+		{"oldfs", no_argument, NULL, 'O'},
 		{0, 0, 0, 0}
 	};
 
@@ -160,7 +161,7 @@ static errno_t fsck_init(fsck_parse_t *data,
 		return USER_ERROR;
 	}
 
-	while ((c = getopt_long(argc, argv, "L:VhnqafU:b:r?dB:plo:c:uy", 
+	while ((c = getopt_long(argc, argv, "L:VhnqafU:b:r?dB:plo:c:uyO", 
 				options, &option_index)) >= 0) 
 	{
 		switch (c) {
@@ -234,6 +235,9 @@ static errno_t fsck_init(fsck_parse_t *data,
 		case 'u':
 			aal_set_bit(&data->options, FSCK_OPT_WHOLE);
 			break;
+		case 'O':
+			aal_set_bit(&data->options, FSCK_OPT_OLD);
+			break;
 		}
 	}
 	
@@ -397,9 +401,9 @@ static errno_t fsck_check_init(repair_data_t *repair,
 	repair_error_count(repair, res);
 	
 	/* Leave device RW if not CHECK mode. */
+	reiser4_journal_sync(repair->fs->journal);
+	reiser4_fs_sync(repair->fs);
 	if (fs_mode == RM_CHECK) {
-		reiser4_journal_sync(repair->fs->journal);
-		reiser4_fs_sync(repair->fs);
 		if (aal_device_reopen(host, host->blksize, flags)) {
 			aal_fatal("Failed to reopen the device RW.");
 			res = -EIO;
@@ -528,6 +532,8 @@ int main(int argc, char *argv[]) {
 	repair.flags = 
 		fsck_opt(&parse_data, FSCK_OPT_DEBUG) << REPAIR_DEBUG |
 		fsck_opt(&parse_data, FSCK_OPT_WHOLE) << REPAIR_WHOLE |
+		fsck_opt(&parse_data, FSCK_OPT_OLD) << REPAIR_WHOLE |
+		fsck_opt(&parse_data, FSCK_OPT_OLD) << REPAIR_NO_MKID |
 		fsck_opt(&parse_data, FSCK_OPT_YES) << REPAIR_YES;
 		
 	repair.bitmap_file = parse_data.bitmap_file;
@@ -570,7 +576,7 @@ int main(int argc, char *argv[]) {
 	/* Report about the results. */
 	if (res < 0 || ex == OPER_ERROR) {
 		ex = OPER_ERROR;
-		fprintf(stderr, "Operational error occured while fscking.\n");
+		fprintf(stderr, "Operational error occurred while fscking.\n");
 		goto free_fsck;
 	} 
 	
